#!/bin/sh

# Link script to link in CRTM release fixfiles into a single directory.
#
# $Id: link_coeffs.sh 29846 2013-07-10 21:06:59Z paul.vandelst@noaa.gov $

script_id()
{
  REVISION='$Revision: 29846 $'
  LAST_CHANGED_DATE='$LastChangedDate: 2013-07-10 17:06:59 -0400 (Wed, 10 Jul 2013) $'
  echo
  echo "${SCRIPT_NAME} ${REVISION} ${LAST_CHANGED_DATE}"
  echo " "`date`
  echo " Support email: NCEP.List.EMC.JCSDA_CRTM.Support@noaa.gov"
}

usage()
{
  echo
  echo " Usage: link_coeffs.sh [-achlx] source-dir dest-dir [sensor_id1 sensor id2 ... sensor_idN]"
  echo
  echo "   Link in CRTM release fixfiles into a single directory."
  echo
  echo " Options:"
  echo "   -a"
  echo "         Link in ODAS TauCoeff files. ODPS is the default."
  echo
  echo "         Note: Currently there are no ODPS TauCoeff files"
  echo "               for visible sensors (we're working on it)."
  echo
  echo "   -c"
  echo "         Perform additional special case linking for the AIRS 281 and"
  echo "         CrIS 399 channel subset datafiles for use with the GSI satinfo"
  echo "         file. The naming convention is:"
  echo "           airs281SUBSET_aqua.[Spc|Tau]Coeff.bin -> airs281_aqua.[Spc|Tau]Coeff.bin"
  echo "           cris_npp.[Spc|Tau]Coeff.bin           -> cris399_aqua.[Spc|Tau]Coeff.bin"
  echo
  echo "   -h"
  echo "         Print this message"
  echo
  echo "   -l"
  echo "         Link in little-endian files. BIG-ENDIAN is the default."
  echo
  echo "   -x"
  echo "         Turn on execution tracing"
  echo
  echo " Arguments:"
  echo "   source-dir"
  echo "         The /fix directory location of the CRTM release."
  echo "         This directory must already exist."
  echo
  echo "   dest-dir"
  echo "         The directory into which the coefficients will be linked."
  echo "         If this directory does not exist, it is created."
  echo
  echo " Optional arguments:"
  echo "   sensor_id1 sensor id2 ... sensor_idN"
  echo "         A list of sensor ids identifying the particular sensors for which"
  echo "         the SpcCoeff and TauCoeff files are required. If not specified, all"
  echo "         available sensor files are linked."
  echo
}

error_message()
{
  MESSAGE=$1
  echo >&2
  echo "  *********************" >&2
  echo "  ${SCRIPT_NAME}(ERROR): ${MESSAGE}" >&2
  echo "  *********************" >&2
}


########################################################################
#                           MAIN SCRIPT BEGINS                         #
########################################################################

# Setup
SCRIPT_NAME=`basename $0`
# ...Defintiions
SUCCESS=0
FAILURE=1
# ..Define defaults
ENDIAN_TYPE="Big_Endian"
TAUCOEFF_TYPE="ODPS"
ALL_SENSORS="yes"
SPECIAL_CASE="no"
# ...Define commands
LINK="ln -sf"
# ...Define helper script, and make sure it can be found
LINK_SCRIPT="linkfiles.sh"
type ${LINK_SCRIPT} >/dev/null 2>&1 || {
  error_message "Cannot find ${LINK_SCRIPT} helper script. Exiting..."
  exit ${FAILURE}
}
# ...Define sensor independent coefficient files.
COMMON_COEFF_FILES="AerosolCoeff.bin \
CloudCoeff.bin \
FASTEM4.MWwater.EmisCoeff.bin \
FASTEM5.MWwater.EmisCoeff.bin \
Nalli.IRwater.EmisCoeff.bin \
WuSmith.IRwater.EmisCoeff.bin \
NPOESS.IRice.EmisCoeff.bin \
NPOESS.IRland.EmisCoeff.bin \
NPOESS.IRsnow.EmisCoeff.bin \
NPOESS.VISice.EmisCoeff.bin \
NPOESS.VISland.EmisCoeff.bin \
NPOESS.VISsnow.EmisCoeff.bin \
NPOESS.VISwater.EmisCoeff.bin \
IGBP.IRland.EmisCoeff.bin \
IGBP.VISland.EmisCoeff.bin \
USGS.IRland.EmisCoeff.bin \
USGS.VISland.EmisCoeff.bin"



# Parse command line options
while getopts :xhlac OPTVAL; do

  # If option argument looks like another option exit the loop
  case ${OPTARG} in
    -*) break;;
  esac

  # Parse the valid options here
  case ${OPTVAL} in
    l)  ENDIAN_TYPE="Little_Endian" ;;
    a)  TAUCOEFF_TYPE="ODAS" ;;
    c)  SPECIAL_CASE="yes";;
    x)  script_id; set -x ;;
    h)  usage | more; exit ${SUCCESS} ;;
    \?) OPTVAL=${OPTARG}; break ;;
  esac
done

# ...Remove the options processed
shift $((OPTIND - 1))

# ...Output invalidities based on OPTVAL
case ${OPTVAL} in

  # If OPTVAL contains nothing, then all options
  # have been successfully parsed.
  # So, check for presence of mandatory arguments.
  \?) if [ $# -lt 2 ]; then
        usage
        error_message "Missing source-dir and/or dest-dir arguments"
        exit ${FAILURE}
      fi;;

  # Invalid option
  ?) usage
     error_message "Invalid option '-${OPTARG}'"
     exit ${FAILURE};;
esac



# Transfer, and remove, the arguments
SOURCE_DIR=$1
DEST_DIR=$2
shift 2


# Check if sensor ids have been specified
if [ $# -ne 0 ]; then
  SENSOR_ID_LIST="$*"
  ALL_SENSORS="no"
fi


# Check the directory arguments

# ...Ensure the source directory exists
if [ ! -d ${SOURCE_DIR} ]; then
  usage
  error_message "Source directory '${SOURCE_DIR}' does not exist. Exiting."
  exit ${FAILURE}
fi

# ...Get the absolute path to the source directory
CURRENT_DIR=${PWD}
cd ${SOURCE_DIR}; SOURCE_DIR=`pwd -L`; cd ${CURRENT_DIR}

# ...Create the destination directory if necessary
if [ ! -d ${DEST_DIR} ]; then
  mkdir ${DEST_DIR}
  if [ $? -ne ${SUCCESS} ]; then
    error_message "Error creating destination directory '${DEST_DIR}'. Exiting."
    exit ${FAILURE}
  fi
fi



# Begin the linking process
echo
echo "Linking coefficient files from root source directory,"
echo "  ${SOURCE_DIR}"
echo "to destination directory,"
echo "  ${DEST_DIR}"

# Go to destination
cd ${DEST_DIR}
if [ $? -ne ${SUCCESS} ]; then
  error_message "Error cd'ing to destination directory '${DEST_DIR}'. Exiting."
  exit ${FAILURE}
fi


# Link common files
echo; echo "...linking sensor-independent coefficient files..."
${LINK_SCRIPT} -d ${ENDIAN_TYPE} ${SOURCE_DIR} ${COMMON_COEFF_FILES}


# Link the SpcCoeff files
echo; echo "...linking SpcCoeff coefficient files..."
if [ "${ALL_SENSORS}" = "yes" ]; then
  SPCCOEFF_FILES=`ls ${SOURCE_DIR}/SpcCoeff/${ENDIAN_TYPE}`
else
  for SENSOR_ID in ${SENSOR_ID_LIST}; do
    SPCCOEFF_FILES="${SPCCOEFF_FILES} ${SENSOR_ID}.SpcCoeff.bin"
  done
fi
${LINK_SCRIPT} -d ${ENDIAN_TYPE} ${SOURCE_DIR} ${SPCCOEFF_FILES}
# ...Perform special case linking if required
if [ "${SPECIAL_CASE}" = "yes" ]; then
  ${LINK} airs281_aqua.SpcCoeff.bin airs281SUBSET_aqua.SpcCoeff.bin
  ${LINK} cris399_npp.SpcCoeff.bin  cris_npp.SpcCoeff.bin
fi

# Link the TauCoeff files
echo; echo "...linking TauCoeff coefficient files..."
if [ "${ALL_SENSORS}" = "yes" ]; then
  TAUCOEFF_FILES=`ls ${SOURCE_DIR}/TauCoeff/${TAUCOEFF_TYPE}/${ENDIAN_TYPE}`
else
  for SENSOR_ID in ${SENSOR_ID_LIST}; do
    TAUCOEFF_FILES="${TAUCOEFF_FILES} ${SENSOR_ID}.TauCoeff.bin"
  done
fi
${LINK_SCRIPT} -d ${ENDIAN_TYPE} ${SOURCE_DIR}/TauCoeff/${TAUCOEFF_TYPE} ${TAUCOEFF_FILES}
# ...Perform special case linking if required
if [ "${SPECIAL_CASE}" = "yes" ]; then
  ${LINK} airs281_aqua.TauCoeff.bin airs281SUBSET_aqua.TauCoeff.bin
  ${LINK} cris399_npp.TauCoeff.bin  cris_npp.TauCoeff.bin
fi

# Return to original directory
cd ${CURRENT_DIR}
